package fr.lip6.mcc.pnml.exporter;

import fr.lip6.mcc.pnml.generators.ClientServerRepetitions;
import fr.lip6.mcc.pnml.generators.DynPhilo;
import fr.lip6.mcc.pnml.generators.GlobalAllocationRes;
import fr.lip6.mcc.pnml.generators.Models;
import fr.lip6.mcc.pnml.generators.Peterson;
import fr.lip6.mcc.pnml.generators.Philo;
import fr.lip6.mcc.pnml.generators.SharedMemory;
import fr.lip6.mcc.pnml.generators.TokenRing;

public class MainExporter {

	private static final String PNML_PATH = "pnmlModels/";
	private static final String CS_PATH = PNML_PATH + "csRepetitions/cs_repetitions-";
	private static final String GAL_PATH = PNML_PATH + "galloc_res/galloc_res-";
	private static final String DYNPHI_PATH = PNML_PATH + "dynphilos/philo_dyn-";
	private static final String PETER_PATH = PNML_PATH + "peterson/Peterson-";
	private static final String TR_PATH = PNML_PATH + "Tokenring/TokenRing-";
	private static final String PHILO_PATH = PNML_PATH + "philos/Philosophers-";
	private static final String SHMEM_PATH = PNML_PATH + "sharedmemory/SharedMemory-";

	/**
	 * @param args
	 */
	public static void main(String[] args) {	
		if(args.length == 0){
			System.err.println("Usage: invoque this program with at least the name of a model to generate.");
			System.err.println("See: fr.lip6.mcc.pnml.generators#Model.java.");
			return;
		}
		final PNMLExporter exporter = new PNMLExporter();
		Models model = null;
		for (String arg : args) {
			model = Models.valueOf(arg);
			switch (model) {
			// Export ClientServer with Repetitions
			case CSREPETITIONS:
				exporter.setGenerator(new ClientServerRepetitions());
				exporter.generate2Pnml(null, CS_PATH, new int[] { 25, 49, 100, 169, 225, 256, 289, 324, 400, 484, 625, 784,
						900 });
				break;
			// Export Dynamic Philosophers
			case DYNPHILO:
				exporter.setGenerator(new DynPhilo());
				exporter.generate2Pnml(null, DYNPHI_PATH, new int[] { 2, 3, 10, 20, 50, 75, 80 });
				break;
			// Export Global Allocation Resource Management
			case GALLOCRES:
				exporter.setGenerator(new GlobalAllocationRes());
				exporter.generate2Pnml(null, GAL_PATH, new int[] { 3, 5, 6, 7, 9, 10, 11 });
				break;
			// Export Peterson
			case PETERSON:
				exporter.setGenerator(new Peterson());
				exporter.generate2Pnml(null, PETER_PATH, new int[] { 2, 3, 4, 5, 6, 7 });
				break;
			// Export Philosophers
			case PHILO:
				exporter.setGenerator(new Philo());
				exporter.generate2Pnml(null, PHILO_PATH, new int[] { 5, 10, 20, 50, 100, 500, 1000, 5000, 10000, 50000,
						100000 });
				break;
			// Export Shared Memory
			case SHMEM:
				exporter.setGenerator(new SharedMemory());
				exporter.generate2Pnml(null, SHMEM_PATH, new int[] { 5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000,
						10000, 20000, 50000, 100000 });
				break;
			// Export Token Ring
			case TOKENRING:
				exporter.setGenerator(new TokenRing());
				exporter.generate2Pnml(null, TR_PATH, new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, 20, 30, 40, 50 });
				break;
			default:
				System.err.println("Unknown model!");
				break;
			}
		}
	}

	private MainExporter() {
		super();
	}
}
