package fr.lip6.mcc.pnml.generators;

import java.util.HashMap;
import java.util.Map;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.booleans.BooleansFactory;
import fr.lip6.move.pnml.symmetricnet.booleans.Inequality;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.CyclicEnumerationsFactory;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.Successor;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FiniteEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.Condition;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ConditionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.Add;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AddHLAPI;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Tuple;
import fr.lip6.move.pnml.symmetricnet.terms.UserOperator;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

@Preamble(
		author = "Lom", 
		date = "21/02/2011",
		description = "Manual construction of the Petri net (ISO/IEC 15909-2) of the Peterson model. \n", 
		provider = "Emmanuel Paviot-Adet"
)
public class Peterson extends RootGenerator implements PnmlGenerator {
	/**
	 * Root doc Class for the net
	 */
	protected HLAPIRootClass peterDoc;

	public Peterson() {
		super();
	}
	
	@Override
	public void generate2Pnml(String proprietaryFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Peterson instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Peterson instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done : Peterson instance " + instanceSize);
	}

	protected void createModel(int instanceSize) {
		peterDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("petersonDoc" + instanceSize, peterDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("peterson", PNTypeHLAPI.SYMNET, new NameHLAPI("Peterson"),
					(PetriNetDocHLAPI) peterDoc);
			PageHLAPI defaultPage = new PageHLAPI("petersonDefaultPage", net);

			/* Net declarations */
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class Process is 0..N
			CyclicEnumerationHLAPI processEnum = new CyclicEnumerationHLAPI();
			Map<Integer, FEConstantHLAPI> procEnumConsts = new HashMap<Integer, FEConstantHLAPI>();
			for (int i = 0; i <= instanceSize; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("process" + String.valueOf(i), String.valueOf(i));
				processEnum.addElementsHLAPI(oneConstant);
				procEnumConsts.put(i, oneConstant);
			}
			NamedSortHLAPI procName = new NamedSortHLAPI("process", "Process", processEnum, declarations);
			// Class Tour is 0..N-1
			CyclicEnumerationHLAPI tourEnum = new CyclicEnumerationHLAPI();
			Map<Integer, FEConstantHLAPI> tourEnumConsts = new HashMap<Integer, FEConstantHLAPI>();
			for (int i = 0; i <= instanceSize - 1; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("tour" + String.valueOf(i), String.valueOf(i));
				tourEnum.addElementsHLAPI(oneConstant);
				tourEnumConsts.put(i, oneConstant);
			}
			NamedSortHLAPI tourName = new NamedSortHLAPI("tour", "Tour", tourEnum, declarations);
			// Class Bool is [T, F]
			FiniteEnumerationHLAPI boolEnum = new FiniteEnumerationHLAPI();
			FEConstantHLAPI falseConst = new FEConstantHLAPI("false", "F", boolEnum);
			FEConstantHLAPI trueConst = new FEConstantHLAPI("true", "T", boolEnum);
			NamedSortHLAPI boolName = new NamedSortHLAPI("bool", "Bool", boolEnum, declarations);
			// Domain ProcTour is <Process, Tour>
			ProductSortHLAPI procTour = new ProductSortHLAPI();
			procTour.addElementSortHLAPI(new UserSortHLAPI(procName));
			procTour.addElementSortHLAPI(new UserSortHLAPI(tourName));
			NamedSortHLAPI procTourName = new NamedSortHLAPI("proctour", "ProcTour", procTour, declarations);
			// Domain TourProc is <Tour, Proc>
			ProductSortHLAPI tourProc = new ProductSortHLAPI();
			tourProc.addElementSortHLAPI(new UserSortHLAPI(tourName));
			tourProc.addElementSortHLAPI(new UserSortHLAPI(procName));
			NamedSortHLAPI tourProcName = new NamedSortHLAPI("tourproc", "TourProc", tourProc, declarations);
			// Domain ProcTourProc is <Process, Tour, Process>
			ProductSortHLAPI procTourProc = new ProductSortHLAPI();
			procTourProc.addElementSortHLAPI(new UserSortHLAPI(procName));
			procTourProc.addElementSortHLAPI(new UserSortHLAPI(tourName));
			procTourProc.addElementSortHLAPI(new UserSortHLAPI(procName));
			NamedSortHLAPI procTourProcName = new NamedSortHLAPI("proctourproc", "ProcTourProc", procTourProc,
					declarations);
			// Domain ProcBool is <Process, Bool>
			ProductSortHLAPI procBool = new ProductSortHLAPI();
			procBool.addElementSortHLAPI(new UserSortHLAPI(procName));
			procBool.addElementSortHLAPI(new UserSortHLAPI(boolName));
			NamedSortHLAPI procBoolName = new NamedSortHLAPI("procbool", "ProcBool", procBool, declarations);
			// Var i, k in Process
			VariableDeclHLAPI iDecl = new VariableDeclHLAPI("vari", "i", new UserSortHLAPI(procName), declarations);
			VariableDeclHLAPI kDecl = new VariableDeclHLAPI("vark", "k", new UserSortHLAPI(procName), declarations);
			// Var j in Tour
			VariableDeclHLAPI jDecl = new VariableDeclHLAPI("varj", "j", new UserSortHLAPI(tourName), declarations);

			/* Places */
			// Place Idle {dom: Process; mark: Process.all}
			PlaceHLAPI idle = new PlaceHLAPI("idle", new NameHLAPI("Idle"), null, new TypeHLAPI(null, "Process",
					new UserSortHLAPI(procName)), new HLMarkingHLAPI(null, "Process.all", new AllHLAPI(
					new UserSortHLAPI(procName))), defaultPage);
			// Place WantSection {dom: ProcBool; mark: <Process.all, F>
			Add add0 = MultisetsFactory.eINSTANCE.createAdd();
			for(int i = 0; i <= instanceSize; i++){
				UserOperator uo0 = TermsFactory.eINSTANCE.createUserOperator();
				uo0.setDeclaration(procEnumConsts.get(i).getContainedItem());
				UserOperator uo1 = TermsFactory.eINSTANCE.createUserOperator();
				uo1.setDeclaration(falseConst.getContainedItem());
				Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
				tuple0.getSubterm().add(uo0);
				tuple0.getSubterm().add(uo1);
				NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
				one0.setValue(1);
				one0.setType(IntegersFactory.eINSTANCE.createPositive());
				NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
				nof0.getSubterm().add(one0);
				nof0.getSubterm().add(tuple0);
				add0.getSubterm().add(nof0);	
			}
			PlaceHLAPI wantSection = new PlaceHLAPI("wantSection", new NameHLAPI("WantSection"), null, new TypeHLAPI(
					null, "ProcBool", new UserSortHLAPI(procBoolName)), new HLMarkingHLAPI(null, "1'(Process.all, F)",
					new AddHLAPI(add0)), defaultPage);
			// Place AskForSection {dom: ProcTour}
			PlaceHLAPI askForSection = new PlaceHLAPI("askForSection", new NameHLAPI("AskForSection"), null,
					new TypeHLAPI(null, "ProcTour", new UserSortHLAPI(procTourName)), null, defaultPage);
			// Place Turn {dom: TourProc, mark: <Tour.all, 0>
			Add add1 = MultisetsFactory.eINSTANCE.createAdd();
			for(int i = 0; i <= instanceSize - 1; i++){
				UserOperator uo0 = TermsFactory.eINSTANCE.createUserOperator();
				uo0.setDeclaration(tourEnumConsts.get(i).getContainedItem());
				UserOperator uo1 = TermsFactory.eINSTANCE.createUserOperator();
				uo1.setDeclaration(procEnumConsts.get(0).getContainedItem());
				Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
				tuple0.getSubterm().add(uo0);
				tuple0.getSubterm().add(uo1);
				NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
				one0.setValue(1);
				one0.setType(IntegersFactory.eINSTANCE.createPositive());
				NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
				nof0.getSubterm().add(one0);
				nof0.getSubterm().add(tuple0);
				add1.getSubterm().add(nof0);	
			}
			PlaceHLAPI turn = new PlaceHLAPI("turn", new NameHLAPI("Turn"), null, new TypeHLAPI(null, "TourProc",
					new UserSortHLAPI(tourProcName)), new HLMarkingHLAPI(null, "1'(Tour.all, 0)",
					new AddHLAPI(add1)), defaultPage);
			// Place TestTurn {dom: ProcTour}
			PlaceHLAPI testTurn = new PlaceHLAPI("testTurn", new NameHLAPI("TestTurn"), null, new TypeHLAPI(null,
					"ProcTour", new UserSortHLAPI(procTourName)), null, defaultPage);
			// Place BeginLoop {dom: ProcTourProc}
			PlaceHLAPI beginLoop = new PlaceHLAPI("beginLoop", new NameHLAPI("BeginLoop"), null, new TypeHLAPI(null,
					"ProcTourProc", new UserSortHLAPI(procTourProcName)), null, defaultPage);
			// Place EndTurn {dom: ProcTour}
			PlaceHLAPI endTurn = new PlaceHLAPI("endTurn", new NameHLAPI("EndTurn"), null, new TypeHLAPI(null,
					"ProcTour", new UserSortHLAPI(procTourName)), null, defaultPage);
			// Place CS {dom: Process}
			PlaceHLAPI cs = new PlaceHLAPI("cs", new NameHLAPI("CS"), null, new TypeHLAPI(null, "Process",
					new UserSortHLAPI(procName)), null, defaultPage);
			// Place TestIdentity {dom :ProcTourProc}
			PlaceHLAPI testIdentity = new PlaceHLAPI("testIdentity", new NameHLAPI("TestIdentity"), null,
					new TypeHLAPI(null, "ProcTourProc", new UserSortHLAPI(procTourProcName)), null, defaultPage);
			// Place TestAlone {dom: ProcTourProc}
			PlaceHLAPI testAlone = new PlaceHLAPI("testAlone", new NameHLAPI("TestAlone"), null, new TypeHLAPI(null,
					"ProcTourProc", new UserSortHLAPI(procTourProcName)), null, defaultPage);
			// Place IsEndLoop {dom: ProcTourProc}
			PlaceHLAPI isEndLoop = new PlaceHLAPI("isEndLoop", new NameHLAPI("IsEndLoop"), null, new TypeHLAPI(null,
					"ProcTourProc", new UserSortHLAPI(procTourProcName)), null, defaultPage);

			/* Transitions */
			// Ask
			TransitionHLAPI ask = new TransitionHLAPI("ask", new NameHLAPI("Ask"), null, null, defaultPage);
			// UpdateTurn
			TransitionHLAPI updateTurn = new TransitionHLAPI("updateTurn", new NameHLAPI("UpdateTurn"), null, null,
					defaultPage);
			// TurnEqual
			TransitionHLAPI turnEqual = new TransitionHLAPI("turnEqual", new NameHLAPI("TurnEqual"), null, null,
					defaultPage);
			// TurnDiff {guard: i <> k}
			Variable vari0 = TermsFactory.eINSTANCE.createVariable();
			vari0.setVariableDecl(iDecl.getContainedItem());
			Variable vark0 = TermsFactory.eINSTANCE.createVariable();
			vark0.setVariableDecl(kDecl.getContainedItem());
			Inequality ineq0 = BooleansFactory.eINSTANCE.createInequality();
			ineq0.getSubterm().add(vari0);
			ineq0.getSubterm().add(vark0);
			Condition turnDiffCond = HlcorestructureFactory.eINSTANCE.createCondition();
			turnDiffCond.setStructure(ineq0);
			turnDiffCond.setText("[i ineq k]");
			ConditionHLAPI turnDiffCondition = new ConditionHLAPI(turnDiffCond);
			TransitionHLAPI turnDiff = new TransitionHLAPI("turnDiff", new NameHLAPI("TurnDiff"), null,
					turnDiffCondition, defaultPage);
			// AccessCS
			TransitionHLAPI accessCS = new TransitionHLAPI("accessCS", new NameHLAPI("AccessCS"), null, null,
					defaultPage);
			// ProgressTurn {guard: j <> N - 1} Accord de Manu.
			Variable varj0 = TermsFactory.eINSTANCE.createVariable();
			varj0.setVariableDecl(jDecl.getContainedItem());
			UserOperator uo2 = TermsFactory.eINSTANCE.createUserOperator();
			uo2.setDeclaration(tourEnumConsts.get(instanceSize - 1).getContainedItem());
			Inequality ineq1 = BooleansFactory.eINSTANCE.createInequality();
			ineq1.getSubterm().add(varj0);
			ineq1.getSubterm().add(uo2);
			Condition progressTurnCond = HlcorestructureFactory.eINSTANCE.createCondition();
			progressTurnCond.setStructure(ineq1);
			progressTurnCond.setText("[j ineq " + String.valueOf(instanceSize - 1) + "]");
			ConditionHLAPI progressTurnCondition = new ConditionHLAPI(progressTurnCond);
			TransitionHLAPI progressTurn = new TransitionHLAPI("progressTurn", new NameHLAPI("ProgressTurn"), null,
					progressTurnCondition, defaultPage);
			// BecomeIdle
			TransitionHLAPI becomeIdle = new TransitionHLAPI("becomeIdle", new NameHLAPI("BecomeIdle"), null, null,
					defaultPage);
			// ContinueLoop
			TransitionHLAPI continueLoop = new TransitionHLAPI("continueLoop", new NameHLAPI("ContinueLoop"), null,
					null, defaultPage);
			// Identity
			TransitionHLAPI identity = new TransitionHLAPI("identity", new NameHLAPI("Identity"), null, null,
					defaultPage);
			// NoIdentity {guard: i <> k}
			Variable vari1 = TermsFactory.eINSTANCE.createVariable();
			vari1.setVariableDecl(iDecl.getContainedItem());
			Variable vark1 = TermsFactory.eINSTANCE.createVariable();
			vark1.setVariableDecl(kDecl.getContainedItem());
			Inequality ineq2 = BooleansFactory.eINSTANCE.createInequality();
			ineq2.getSubterm().add(vari1);
			ineq2.getSubterm().add(vark1);
			Condition noIdentityCond = HlcorestructureFactory.eINSTANCE.createCondition();
			noIdentityCond.setStructure(ineq2);
			noIdentityCond.setText("[i ineq k]");
			ConditionHLAPI noIdentityCondition = new ConditionHLAPI(noIdentityCond);
			TransitionHLAPI noIdentity = new TransitionHLAPI("noIdentity", new NameHLAPI("NoIdentity"), null,
					noIdentityCondition, defaultPage);
			// Loop {guard: k<> N}. Accord de Manu
			Variable vark2 = TermsFactory.eINSTANCE.createVariable();
			vark2.setVariableDecl(kDecl.getContainedItem());
			UserOperator uo3 = TermsFactory.eINSTANCE.createUserOperator();
			uo3.setDeclaration(procEnumConsts.get(instanceSize).getContainedItem());
			Inequality ineq3 = BooleansFactory.eINSTANCE.createInequality();
			ineq3.getSubterm().add(vark2);
			ineq3.getSubterm().add(uo3);
			Condition loopCond = HlcorestructureFactory.eINSTANCE.createCondition();
			loopCond.setStructure(ineq3);
			loopCond.setText("[k ineq " + String.valueOf(instanceSize) + "]");
			ConditionHLAPI loopCondition = new ConditionHLAPI(loopCond);
			TransitionHLAPI loop = new TransitionHLAPI("loop", new NameHLAPI("Loop"), null, loopCondition, defaultPage);
			// NotAlone
			TransitionHLAPI notAlone = new TransitionHLAPI("notAlone", new NameHLAPI("NotAlone"), null, null,
					defaultPage);
			// Alone1
			TransitionHLAPI alone1 = new TransitionHLAPI("alone1", new NameHLAPI("Alone1"), null, null, defaultPage);
			// EndLoop
			TransitionHLAPI endLoop = new TransitionHLAPI("endLoop", new NameHLAPI("EndLoop"), null, null, defaultPage);

			/* Arcs */
			// Idle -> Ask : 1'(i)
			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari2 = TermsFactory.eINSTANCE.createVariable();
			vari2.setVariableDecl(iDecl.getContainedItem());
			NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof0.getSubterm().add(one2);
			nof0.getSubterm().add(vari2);
			HLAnnotation idle2askhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			idle2askhl.setStructure(nof0);
			idle2askhl.setText("1'(i)");
			ArcHLAPI idle2ask = new ArcHLAPI("idle2ask", new NameHLAPI("Idle2Ask"), idle, ask, null,
					new HLAnnotationHLAPI(idle2askhl), defaultPage);
			// BecomeIdle --> Idle : 1'(i)
			NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari3 = TermsFactory.eINSTANCE.createVariable();
			vari3.setVariableDecl(iDecl.getContainedItem());
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one3);
			nof1.getSubterm().add(vari3);
			HLAnnotation bci2idlehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			bci2idlehl.setStructure(nof1);
			bci2idlehl.setText("1'(i)");
			ArcHLAPI bci2idle = new ArcHLAPI("becomeidle2idle", new NameHLAPI("BecomeIdle2Idle"), becomeIdle, idle,
					null, new HLAnnotationHLAPI(bci2idlehl), defaultPage);
			// WantSection --> Ask : 1'(i,F)
			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari4 = TermsFactory.eINSTANCE.createVariable();
			vari4.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo4 = TermsFactory.eINSTANCE.createUserOperator();
			uo4.setDeclaration(falseConst.getContainedItem());
			Tuple tuple2 = TermsFactory.eINSTANCE.createTuple();
			tuple2.getSubterm().add(vari4);
			tuple2.getSubterm().add(uo4);
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one4);
			nof2.getSubterm().add(tuple2);
			HLAnnotation wantSection2Askhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			wantSection2Askhl.setStructure(nof2);
			wantSection2Askhl.setText("1'(i,F)");
			ArcHLAPI wantSection2Ask = new ArcHLAPI("wantSection2ask", new NameHLAPI("WantSection2Ask"), wantSection,
					ask, null, new HLAnnotationHLAPI(wantSection2Askhl), defaultPage);
			// Ask --> WantSection : 1'(i,T)
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari5 = TermsFactory.eINSTANCE.createVariable();
			vari5.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo5 = TermsFactory.eINSTANCE.createUserOperator();
			uo5.setDeclaration(trueConst.getContainedItem());
			Tuple tuple3 = TermsFactory.eINSTANCE.createTuple();
			tuple3.getSubterm().add(vari5);
			tuple3.getSubterm().add(uo5);
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one5);
			nof3.getSubterm().add(tuple3);
			HLAnnotation ask2wantSectionhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			ask2wantSectionhl.setStructure(nof3);
			ask2wantSectionhl.setText("1'(i,T)");
			ArcHLAPI ask2wantSection = new ArcHLAPI("ask2wantSection", new NameHLAPI("Ask2WantSection"), ask,
					wantSection, null, new HLAnnotationHLAPI(ask2wantSectionhl), defaultPage);
			// WantSection --> BecomeIdle : 1'(i,T)
			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari6 = TermsFactory.eINSTANCE.createVariable();
			vari6.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo6 = TermsFactory.eINSTANCE.createUserOperator();
			uo6.setDeclaration(trueConst.getContainedItem());
			Tuple tuple4 = TermsFactory.eINSTANCE.createTuple();
			tuple4.getSubterm().add(vari6);
			tuple4.getSubterm().add(uo6);
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one6);
			nof4.getSubterm().add(tuple4);
			HLAnnotation wantSection2becomeIdlehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			wantSection2becomeIdlehl.setStructure(nof4);
			wantSection2becomeIdlehl.setText("1'(i,T)");
			ArcHLAPI wantSection2becomeIdle = new ArcHLAPI("wantSection2becomeIdle", new NameHLAPI(
					"WantSection2BecomeIdle"), wantSection, becomeIdle, null, new HLAnnotationHLAPI(
					wantSection2becomeIdlehl), defaultPage);
			// BecomeIdle --> WantSection : 1'(i, F)
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari7 = TermsFactory.eINSTANCE.createVariable();
			vari7.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo7 = TermsFactory.eINSTANCE.createUserOperator();
			uo7.setDeclaration(falseConst.getContainedItem());
			Tuple tuple5 = TermsFactory.eINSTANCE.createTuple();
			tuple5.getSubterm().add(vari7);
			tuple5.getSubterm().add(uo7);
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(one7);
			nof5.getSubterm().add(tuple5);
			HLAnnotation becomeIdle2wantSectionhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			becomeIdle2wantSectionhl.setStructure(nof5);
			becomeIdle2wantSectionhl.setText("1'(i,F)");
			ArcHLAPI becomeIdle2wantSection = new ArcHLAPI("becomeIdle2wantSection", new NameHLAPI(
					"BecomeIdle2WantSection"), becomeIdle, wantSection, null, new HLAnnotationHLAPI(
					becomeIdle2wantSectionhl), defaultPage);
			// AskForSection --> UpdateTurn : 1'(i,j)
			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari8 = TermsFactory.eINSTANCE.createVariable();
			vari8.setVariableDecl(iDecl.getContainedItem());
			Variable varj1 = TermsFactory.eINSTANCE.createVariable();
			varj1.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple6 = TermsFactory.eINSTANCE.createTuple();
			tuple6.getSubterm().add(vari8);
			tuple6.getSubterm().add(varj1);
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(one8);
			nof6.getSubterm().add(tuple6);
			HLAnnotation askForSection2updateTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			askForSection2updateTurnhl.setStructure(nof6);
			askForSection2updateTurnhl.setText("1'(i,j)");
			ArcHLAPI askForSection2updateTurn = new ArcHLAPI("askForSection2updateTurn", new NameHLAPI(
					"AskForSection2UpdateTurn"), askForSection, updateTurn, null, new HLAnnotationHLAPI(
					askForSection2updateTurnhl), defaultPage);
			// Ask --> AskForSection : 1'(i,0)
			NumberConstant one9 = IntegersFactory.eINSTANCE.createNumberConstant();
			one9.setValue(1);
			one9.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari9 = TermsFactory.eINSTANCE.createVariable();
			vari9.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo8 = TermsFactory.eINSTANCE.createUserOperator();
			uo8.setDeclaration(tourEnumConsts.get(0).getContainedItem());
			Tuple tuple7 = TermsFactory.eINSTANCE.createTuple();
			tuple7.getSubterm().add(vari9);
			tuple7.getSubterm().add(uo8);
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(one9);
			nof7.getSubterm().add(tuple7);
			HLAnnotation ask2askForSectionhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			ask2askForSectionhl.setStructure(nof7);
			ask2askForSectionhl.setText("1'(i,0)");
			ArcHLAPI ask2askForSection = new ArcHLAPI("ask2askForSection", new NameHLAPI("Ask2askForSection"), ask,
					askForSection, null, new HLAnnotationHLAPI(ask2askForSectionhl), defaultPage);
			// progressTurn --> AskForSection : 1'(i,j++1)
			NumberConstant one10 = IntegersFactory.eINSTANCE.createNumberConstant();
			one10.setValue(1);
			one10.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari10 = TermsFactory.eINSTANCE.createVariable();
			vari10.setVariableDecl(iDecl.getContainedItem());
			Variable varj2 = TermsFactory.eINSTANCE.createVariable();
			varj2.setVariableDecl(jDecl.getContainedItem());
			Successor varj2Succ = CyclicEnumerationsFactory.eINSTANCE.createSuccessor();
			varj2Succ.getSubterm().add(varj2);
			Tuple tuple8 = TermsFactory.eINSTANCE.createTuple();
			tuple8.getSubterm().add(vari10);
			tuple8.getSubterm().add(varj2Succ);
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one10);
			nof8.getSubterm().add(tuple8);
			HLAnnotation progressTurn2askForSectionhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			progressTurn2askForSectionhl.setStructure(nof8);
			progressTurn2askForSectionhl.setText("1'(i,j++1)");
			ArcHLAPI progressTurn2askForSection = new ArcHLAPI("progressTurn2askForSection", new NameHLAPI(
					"ProgressTurn2AskForSection"), progressTurn, askForSection, null, new HLAnnotationHLAPI(
					progressTurn2askForSectionhl), defaultPage);
			// Turn --> UpdateTurn : 1'(j,k)
			NumberConstant one11 = IntegersFactory.eINSTANCE.createNumberConstant();
			one11.setValue(1);
			one11.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varj3 = TermsFactory.eINSTANCE.createVariable();
			varj3.setVariableDecl(jDecl.getContainedItem());
			Variable vark3 = TermsFactory.eINSTANCE.createVariable();
			vark3.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple9 = TermsFactory.eINSTANCE.createTuple();
			tuple9.getSubterm().add(varj3);
			tuple9.getSubterm().add(vark3);
			NumberOf nof9 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof9.getSubterm().add(one11);
			nof9.getSubterm().add(tuple9);
			HLAnnotation turn2updateTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turn2updateTurnhl.setStructure(nof9);
			turn2updateTurnhl.setText("1'(j,k)");
			ArcHLAPI turn2updateTurn = new ArcHLAPI("turn2updateTurn", new NameHLAPI("Turn2updateTurn"), turn,
					updateTurn, null, new HLAnnotationHLAPI(turn2updateTurnhl), defaultPage);
			// UpdateTurn --> Turn : 1'(j,i)
			NumberConstant one12 = IntegersFactory.eINSTANCE.createNumberConstant();
			one12.setValue(1);
			one12.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varj4 = TermsFactory.eINSTANCE.createVariable();
			varj4.setVariableDecl(jDecl.getContainedItem());
			Variable vari11 = TermsFactory.eINSTANCE.createVariable();
			vari11.setVariableDecl(iDecl.getContainedItem());
			Tuple tuple10 = TermsFactory.eINSTANCE.createTuple();
			tuple10.getSubterm().add(varj4);
			tuple10.getSubterm().add(vari11);
			NumberOf nof10 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof10.getSubterm().add(one12);
			nof10.getSubterm().add(tuple10);
			HLAnnotation updateTurn2Turnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			updateTurn2Turnhl.setStructure(nof10);
			updateTurn2Turnhl.setText("1'(j,i)");
			ArcHLAPI updateTurn2Turn = new ArcHLAPI("updateTurn2turn", new NameHLAPI("UpdateTurn2Turn"), updateTurn,
					turn, null, new HLAnnotationHLAPI(updateTurn2Turnhl), defaultPage);
			// Turn --> TurnDiff : 1'(j,k)
			NumberConstant one13 = IntegersFactory.eINSTANCE.createNumberConstant();
			one13.setValue(1);
			one13.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varj5 = TermsFactory.eINSTANCE.createVariable();
			varj5.setVariableDecl(jDecl.getContainedItem());
			Variable vark4 = TermsFactory.eINSTANCE.createVariable();
			vark4.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple11 = TermsFactory.eINSTANCE.createTuple();
			tuple11.getSubterm().add(varj5);
			tuple11.getSubterm().add(vark4);
			NumberOf nof11 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof11.getSubterm().add(one13);
			nof11.getSubterm().add(tuple11);
			HLAnnotation turn2turnDiffhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turn2turnDiffhl.setStructure(nof11);
			turn2turnDiffhl.setText("1'(j,k)");
			ArcHLAPI turn2turnDiff = new ArcHLAPI("turn2turnDiff", new NameHLAPI("Turn2TurnDiff"), turn, turnDiff,
					null, new HLAnnotationHLAPI(turn2turnDiffhl), defaultPage);
			// TurnDiff --> Turn : 1'(j,k)
			NumberConstant one14 = IntegersFactory.eINSTANCE.createNumberConstant();
			one14.setValue(1);
			one14.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varj6 = TermsFactory.eINSTANCE.createVariable();
			varj6.setVariableDecl(jDecl.getContainedItem());
			Variable vark5 = TermsFactory.eINSTANCE.createVariable();
			vark5.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple12 = TermsFactory.eINSTANCE.createTuple();
			tuple12.getSubterm().add(varj6);
			tuple12.getSubterm().add(vark5);
			NumberOf nof12 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof12.getSubterm().add(one14);
			nof12.getSubterm().add(tuple12);
			HLAnnotation turnDiff2turnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turnDiff2turnhl.setStructure(nof12);
			turnDiff2turnhl.setText("1'(j,k)");
			ArcHLAPI turnDiff2turn = new ArcHLAPI("turnDiff2turn", new NameHLAPI("TurnDiff2turn"), turnDiff, turn,
					null, new HLAnnotationHLAPI(turnDiff2turnhl), defaultPage);
			// Turn --> TurnEqual : 1'(j,i)
			NumberConstant one15 = IntegersFactory.eINSTANCE.createNumberConstant();
			one15.setValue(1);
			one15.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varj7 = TermsFactory.eINSTANCE.createVariable();
			varj7.setVariableDecl(jDecl.getContainedItem());
			Variable vari12 = TermsFactory.eINSTANCE.createVariable();
			vari12.setVariableDecl(iDecl.getContainedItem());
			Tuple tuple13 = TermsFactory.eINSTANCE.createTuple();
			tuple13.getSubterm().add(varj7);
			tuple13.getSubterm().add(vari12);
			NumberOf nof13 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof13.getSubterm().add(one15);
			nof13.getSubterm().add(tuple13);
			HLAnnotation turn2turnEqualhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turn2turnEqualhl.setStructure(nof13);
			turn2turnEqualhl.setText("1'(j,i)");
			ArcHLAPI turn2turnEqual = new ArcHLAPI("turn2turnEqual", new NameHLAPI("Turn2TurnEqual"), turn, turnEqual,
					null, new HLAnnotationHLAPI(turn2turnEqualhl), defaultPage);
			// TurnEqual --> Turn : 1'(j,i)
			NumberConstant one16 = IntegersFactory.eINSTANCE.createNumberConstant();
			one16.setValue(1);
			one16.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varj8 = TermsFactory.eINSTANCE.createVariable();
			varj8.setVariableDecl(jDecl.getContainedItem());
			Variable vari13 = TermsFactory.eINSTANCE.createVariable();
			vari13.setVariableDecl(iDecl.getContainedItem());
			Tuple tuple14 = TermsFactory.eINSTANCE.createTuple();
			tuple14.getSubterm().add(varj8);
			tuple14.getSubterm().add(vari13);
			NumberOf nof14 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof14.getSubterm().add(one16);
			nof14.getSubterm().add(tuple14);
			HLAnnotation turnEqual2turnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turnEqual2turnhl.setStructure(nof14);
			turnEqual2turnhl.setText("1'(j,i)");
			ArcHLAPI turnEqual2turn = new ArcHLAPI("turnEqual2turn", new NameHLAPI("TurnEqual2Turn"), turnEqual, turn,
					null, new HLAnnotationHLAPI(turnEqual2turnhl), defaultPage);
			// TestTurn --> TurnEqual : 1'(i,j)
			NumberConstant one17 = IntegersFactory.eINSTANCE.createNumberConstant();
			one17.setValue(1);
			one17.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari14 = TermsFactory.eINSTANCE.createVariable();
			vari14.setVariableDecl(iDecl.getContainedItem());
			Variable varj9 = TermsFactory.eINSTANCE.createVariable();
			varj9.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple15 = TermsFactory.eINSTANCE.createTuple();
			tuple15.getSubterm().add(vari14);
			tuple15.getSubterm().add(varj9);
			NumberOf nof15 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof15.getSubterm().add(one17);
			nof15.getSubterm().add(tuple15);
			HLAnnotation testTurn2turnEqualhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			testTurn2turnEqualhl.setStructure(nof15);
			testTurn2turnEqualhl.setText("1'(i,j)");
			ArcHLAPI testTurn2turnEqual = new ArcHLAPI("testTurn2turnEqual", new NameHLAPI("TestTurn2turnEqual"),
					testTurn, turnEqual, null, new HLAnnotationHLAPI(testTurn2turnEqualhl), defaultPage);
			// TestTurn --> TurnDiff : 1'(i,j)
			NumberConstant one18 = IntegersFactory.eINSTANCE.createNumberConstant();
			one18.setValue(1);
			one18.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari15 = TermsFactory.eINSTANCE.createVariable();
			vari15.setVariableDecl(iDecl.getContainedItem());
			Variable varj10 = TermsFactory.eINSTANCE.createVariable();
			varj10.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple16 = TermsFactory.eINSTANCE.createTuple();
			tuple16.getSubterm().add(vari15);
			tuple16.getSubterm().add(varj10);
			NumberOf nof16 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof16.getSubterm().add(one18);
			nof16.getSubterm().add(tuple16);
			HLAnnotation testTurn2turnDiffhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			testTurn2turnDiffhl.setStructure(nof16);
			testTurn2turnDiffhl.setText("1'(i,j)");
			ArcHLAPI testTurn2turnDiff = new ArcHLAPI("testTurn2turnDiff", new NameHLAPI("TestTurn2TurnDiff"),
					testTurn, turnDiff, null, new HLAnnotationHLAPI(testTurn2turnDiffhl), defaultPage);
			// Not Alone --> TestTurn : 1'(i,j)
			NumberConstant one19 = IntegersFactory.eINSTANCE.createNumberConstant();
			one19.setValue(1);
			one19.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari16 = TermsFactory.eINSTANCE.createVariable();
			vari16.setVariableDecl(iDecl.getContainedItem());
			Variable varj11 = TermsFactory.eINSTANCE.createVariable();
			varj11.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple17 = TermsFactory.eINSTANCE.createTuple();
			tuple17.getSubterm().add(vari16);
			tuple17.getSubterm().add(varj11);
			NumberOf nof17 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof17.getSubterm().add(one19);
			nof17.getSubterm().add(tuple17);
			HLAnnotation notAlone2TestTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			notAlone2TestTurnhl.setStructure(nof17);
			notAlone2TestTurnhl.setText("1'(i,j)");
			ArcHLAPI notAlone2testTurn = new ArcHLAPI("notAlone2testTurn", new NameHLAPI("NotAlone2TestTurn"),
					notAlone, testTurn, null, new HLAnnotationHLAPI(notAlone2TestTurnhl), defaultPage);
			// UpdateTurn --> TestTurn : 1'(i,j)
			NumberConstant one20 = IntegersFactory.eINSTANCE.createNumberConstant();
			one20.setValue(1);
			one20.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari17 = TermsFactory.eINSTANCE.createVariable();
			vari17.setVariableDecl(iDecl.getContainedItem());
			Variable varj12 = TermsFactory.eINSTANCE.createVariable();
			varj12.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple18 = TermsFactory.eINSTANCE.createTuple();
			tuple18.getSubterm().add(vari17);
			tuple18.getSubterm().add(varj12);
			NumberOf nof18 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof18.getSubterm().add(one20);
			nof18.getSubterm().add(tuple18);
			HLAnnotation updateTurn2testTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			updateTurn2testTurnhl.setStructure(nof18);
			updateTurn2testTurnhl.setText("1'(i,j)");
			ArcHLAPI updateTurn2testTurn = new ArcHLAPI("updateTurn2testTurn", new NameHLAPI("UpdateTurn2TestTurn"),
					updateTurn, testTurn, null, new HLAnnotationHLAPI(updateTurn2testTurnhl), defaultPage);
			// BeginLoop --> ContinueLoop : 1'(i,j,k)
			NumberConstant one21 = IntegersFactory.eINSTANCE.createNumberConstant();
			one21.setValue(1);
			one21.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari18 = TermsFactory.eINSTANCE.createVariable();
			vari18.setVariableDecl(iDecl.getContainedItem());
			Variable varj13 = TermsFactory.eINSTANCE.createVariable();
			varj13.setVariableDecl(jDecl.getContainedItem());
			Variable vark6 = TermsFactory.eINSTANCE.createVariable();
			vark6.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple19 = TermsFactory.eINSTANCE.createTuple();
			tuple19.getSubterm().add(vari18);
			tuple19.getSubterm().add(varj13);
			tuple19.getSubterm().add(vark6);
			NumberOf nof19 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof19.getSubterm().add(one21);
			nof19.getSubterm().add(tuple19);
			HLAnnotation beginLoop2continueLoophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			beginLoop2continueLoophl.setStructure(nof19);
			beginLoop2continueLoophl.setText("1'(i,j,k)");
			ArcHLAPI beginLoop2continueLoop = new ArcHLAPI("beginLoop2continueLoop", new NameHLAPI(
					"BeginLoop2continueLoop"), beginLoop, continueLoop, null, new HLAnnotationHLAPI(
					beginLoop2continueLoophl), defaultPage);
			// TurnEqual --> BeginLoop : 1'(i,j,0)
			NumberConstant one22 = IntegersFactory.eINSTANCE.createNumberConstant();
			one22.setValue(1);
			one22.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari19 = TermsFactory.eINSTANCE.createVariable();
			vari19.setVariableDecl(iDecl.getContainedItem());
			Variable varj14 = TermsFactory.eINSTANCE.createVariable();
			varj14.setVariableDecl(jDecl.getContainedItem());
			UserOperator uo9 = TermsFactory.eINSTANCE.createUserOperator();
			uo9.setDeclaration(procEnumConsts.get(0).getContainedItem());
			Tuple tuple20 = TermsFactory.eINSTANCE.createTuple();
			tuple20.getSubterm().add(vari19);
			tuple20.getSubterm().add(varj14);
			tuple20.getSubterm().add(uo9);
			NumberOf nof20 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof20.getSubterm().add(one22);
			nof20.getSubterm().add(tuple20);
			HLAnnotation turnEqual2beginLoophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turnEqual2beginLoophl.setStructure(nof20);
			turnEqual2beginLoophl.setText("1'(i,j,0)");
			ArcHLAPI turnEqual2beginLoop = new ArcHLAPI("turnEqual2beginLoop", new NameHLAPI("TurnEqual2BeginLoop"),
					turnEqual, beginLoop, null, new HLAnnotationHLAPI(turnEqual2beginLoophl), defaultPage);
			// Loop --> BeginLoop : 1'(i,j,k++1)
			NumberConstant one23 = IntegersFactory.eINSTANCE.createNumberConstant();
			one23.setValue(1);
			one23.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari20 = TermsFactory.eINSTANCE.createVariable();
			vari20.setVariableDecl(iDecl.getContainedItem());
			Variable varj15 = TermsFactory.eINSTANCE.createVariable();
			varj15.setVariableDecl(jDecl.getContainedItem());
			Variable vark7 = TermsFactory.eINSTANCE.createVariable();
			vark7.setVariableDecl(kDecl.getContainedItem());
			Successor vark7Succ = CyclicEnumerationsFactory.eINSTANCE.createSuccessor();
			vark7Succ.getSubterm().add(vark7);
			Tuple tuple21 = TermsFactory.eINSTANCE.createTuple();
			tuple21.getSubterm().add(vari20);
			tuple21.getSubterm().add(varj15);
			tuple21.getSubterm().add(vark7Succ);
			NumberOf nof21 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof21.getSubterm().add(one23);
			nof21.getSubterm().add(tuple21);
			HLAnnotation loop2beginLoophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			loop2beginLoophl.setStructure(nof21);
			loop2beginLoophl.setText("1'(i,j,k++1)");
			ArcHLAPI loop2beginLoop = new ArcHLAPI("loop2beginLoop", new NameHLAPI("Loop2beginLoop"), loop, beginLoop,
					null, new HLAnnotationHLAPI(loop2beginLoophl), defaultPage);
			// EndTurn --> ProgressTurn : 1'(i,j)
			NumberConstant one24 = IntegersFactory.eINSTANCE.createNumberConstant();
			one24.setValue(1);
			one24.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari21 = TermsFactory.eINSTANCE.createVariable();
			vari21.setVariableDecl(iDecl.getContainedItem());
			Variable varj16 = TermsFactory.eINSTANCE.createVariable();
			varj16.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple22 = TermsFactory.eINSTANCE.createTuple();
			tuple22.getSubterm().add(vari21);
			tuple22.getSubterm().add(varj16);
			NumberOf nof22 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof22.getSubterm().add(one24);
			nof22.getSubterm().add(tuple22);
			HLAnnotation endTurn2progressTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			endTurn2progressTurnhl.setStructure(nof22);
			endTurn2progressTurnhl.setText("1'(i,j)");
			ArcHLAPI endTurn2progressTurn = new ArcHLAPI("endTurn2progressTurn", new NameHLAPI("EndTurn2ProgressTurn"),
					endTurn, progressTurn, null, new HLAnnotationHLAPI(endTurn2progressTurnhl), defaultPage);
			// EndTurn --> AccessCS : 1'(i, N-1)
			NumberConstant one25 = IntegersFactory.eINSTANCE.createNumberConstant();
			one25.setValue(1);
			one25.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari22 = TermsFactory.eINSTANCE.createVariable();
			vari22.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo10 = TermsFactory.eINSTANCE.createUserOperator();
			uo10.setDeclaration(tourEnumConsts.get(instanceSize - 1).getContainedItem());
			Tuple tuple23 = TermsFactory.eINSTANCE.createTuple();
			tuple23.getSubterm().add(vari22);
			tuple23.getSubterm().add(uo10);
			NumberOf nof23 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof23.getSubterm().add(one25);
			nof23.getSubterm().add(tuple23);
			HLAnnotation endTurn2accessCShl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			endTurn2accessCShl.setStructure(nof23);
			endTurn2accessCShl.setText("1'(i," + String.valueOf(instanceSize - 1) + ")");
			ArcHLAPI endTurn2accessCS = new ArcHLAPI("endTurn2accessCS", new NameHLAPI("endTurn2accessCS"), endTurn,
					accessCS, null, new HLAnnotationHLAPI(endTurn2accessCShl), defaultPage);
			// TurnDiff --> EndTurn : 1'(i,j)
			NumberConstant one26 = IntegersFactory.eINSTANCE.createNumberConstant();
			one26.setValue(1);
			one26.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari23 = TermsFactory.eINSTANCE.createVariable();
			vari23.setVariableDecl(iDecl.getContainedItem());
			Variable varj17 = TermsFactory.eINSTANCE.createVariable();
			varj17.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple24 = TermsFactory.eINSTANCE.createTuple();
			tuple24.getSubterm().add(vari23);
			tuple24.getSubterm().add(varj17);
			NumberOf nof24 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof24.getSubterm().add(one26);
			nof24.getSubterm().add(tuple24);
			HLAnnotation turnDiff2endTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			turnDiff2endTurnhl.setStructure(nof24);
			turnDiff2endTurnhl.setText("1'(i,j)");
			ArcHLAPI turnDiff2endTurn = new ArcHLAPI("turnDiff2endTurn", new NameHLAPI("TurnDiff2EndTurn"), turnDiff,
					endTurn, null, new HLAnnotationHLAPI(turnDiff2endTurnhl), defaultPage);
			// EndLoop --> EndTurn : 1'(i,j)
			NumberConstant one27 = IntegersFactory.eINSTANCE.createNumberConstant();
			one27.setValue(1);
			one27.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari24 = TermsFactory.eINSTANCE.createVariable();
			vari24.setVariableDecl(iDecl.getContainedItem());
			Variable varj18 = TermsFactory.eINSTANCE.createVariable();
			varj18.setVariableDecl(jDecl.getContainedItem());
			Tuple tuple25 = TermsFactory.eINSTANCE.createTuple();
			tuple25.getSubterm().add(vari24);
			tuple25.getSubterm().add(varj18);
			NumberOf nof25 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof25.getSubterm().add(one27);
			nof25.getSubterm().add(tuple25);
			HLAnnotation endLoop2endTurnhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			endLoop2endTurnhl.setStructure(nof25);
			endLoop2endTurnhl.setText("1'(i,j)");
			ArcHLAPI endLoop2endTurn = new ArcHLAPI("endLoop2endTurn", new NameHLAPI("EndLoop2EndTurn"), endLoop,
					endTurn, null, new HLAnnotationHLAPI(endLoop2endTurnhl), defaultPage);
			// TestIdentity --> NoIdentity : 1'(i,j,k)
			NumberConstant one28 = IntegersFactory.eINSTANCE.createNumberConstant();
			one28.setValue(1);
			one28.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari25 = TermsFactory.eINSTANCE.createVariable();
			vari25.setVariableDecl(iDecl.getContainedItem());
			Variable varj19 = TermsFactory.eINSTANCE.createVariable();
			varj19.setVariableDecl(jDecl.getContainedItem());
			Variable vark8 = TermsFactory.eINSTANCE.createVariable();
			vark8.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple26 = TermsFactory.eINSTANCE.createTuple();
			tuple26.getSubterm().add(vari25);
			tuple26.getSubterm().add(varj19);
			tuple26.getSubterm().add(vark8);
			NumberOf nof26 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof26.getSubterm().add(one28);
			nof26.getSubterm().add(tuple26);
			HLAnnotation testIdentity2noIdentityhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			testIdentity2noIdentityhl.setStructure(nof26);
			testIdentity2noIdentityhl.setText("1'(i,j,k)");
			ArcHLAPI testIdentity2noIdentity = new ArcHLAPI("testIdentity2noIdentity", new NameHLAPI(
					"TestIdentity2NoIdentity"), testIdentity, noIdentity, null, new HLAnnotationHLAPI(
					testIdentity2noIdentityhl), defaultPage);
			// TestIdentity --> Identity : 1'(i,j,i)
			NumberConstant one29 = IntegersFactory.eINSTANCE.createNumberConstant();
			one29.setValue(1);
			one29.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari26 = TermsFactory.eINSTANCE.createVariable();
			vari26.setVariableDecl(iDecl.getContainedItem());
			Variable varj20 = TermsFactory.eINSTANCE.createVariable();
			varj20.setVariableDecl(jDecl.getContainedItem());
			Variable vari27 = TermsFactory.eINSTANCE.createVariable();
			vari27.setVariableDecl(iDecl.getContainedItem());
			Tuple tuple27 = TermsFactory.eINSTANCE.createTuple();
			tuple27.getSubterm().add(vari26);
			tuple27.getSubterm().add(varj20);
			tuple27.getSubterm().add(vari27);
			NumberOf nof27 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof27.getSubterm().add(one29);
			nof27.getSubterm().add(tuple27);
			HLAnnotation testIdentity2identityhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			testIdentity2identityhl.setStructure(nof27);
			testIdentity2identityhl.setText("1'(i,j,i)");
			ArcHLAPI testIdentity2identity = new ArcHLAPI("testIdentity2identity", new NameHLAPI(
					"TestIdentity2Identity"), testIdentity, identity, null, new HLAnnotationHLAPI(
					testIdentity2identityhl), defaultPage);
			// ContinueLoop --> TestIdentity : 1'(i,j,k)
			NumberConstant one30 = IntegersFactory.eINSTANCE.createNumberConstant();
			one30.setValue(1);
			one30.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari28 = TermsFactory.eINSTANCE.createVariable();
			vari28.setVariableDecl(iDecl.getContainedItem());
			Variable varj21 = TermsFactory.eINSTANCE.createVariable();
			varj21.setVariableDecl(jDecl.getContainedItem());
			Variable vark9 = TermsFactory.eINSTANCE.createVariable();
			vark9.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple28 = TermsFactory.eINSTANCE.createTuple();
			tuple28.getSubterm().add(vari28);
			tuple28.getSubterm().add(varj21);
			tuple28.getSubterm().add(vark9);
			NumberOf nof28 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof28.getSubterm().add(one30);
			nof28.getSubterm().add(tuple28);
			HLAnnotation continueLoop2testIdentityhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			continueLoop2testIdentityhl.setStructure(nof28);
			continueLoop2testIdentityhl.setText("1'(i,j,k)");
			ArcHLAPI continueLoop2testIdentity = new ArcHLAPI("continueLoop2testIdentity", new NameHLAPI(
					"ContinueLoop2TestIdentity"), continueLoop, testIdentity, null, new HLAnnotationHLAPI(
					continueLoop2testIdentityhl), defaultPage);
			// CS --> BecomeIdle 1'(i)
			NumberConstant one31 = IntegersFactory.eINSTANCE.createNumberConstant();
			one31.setValue(1);
			one31.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari29 = TermsFactory.eINSTANCE.createVariable();
			vari29.setVariableDecl(iDecl.getContainedItem());
			NumberOf nof29 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof29.getSubterm().add(one31);
			nof29.getSubterm().add(vari29);
			HLAnnotation cs2becomeIdlehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			cs2becomeIdlehl.setStructure(nof29);
			cs2becomeIdlehl.setText("1'(i)");
			ArcHLAPI cs2becomeIdle = new ArcHLAPI("cs2becomeIdle", new NameHLAPI("CS2BecomeIdle"), cs, becomeIdle,
					null, new HLAnnotationHLAPI(cs2becomeIdlehl), defaultPage);
			// AccessCS --> CS : 1'(i)
			NumberConstant one32 = IntegersFactory.eINSTANCE.createNumberConstant();
			one32.setValue(1);
			one32.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari30 = TermsFactory.eINSTANCE.createVariable();
			vari30.setVariableDecl(iDecl.getContainedItem());
			NumberOf nof30 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof30.getSubterm().add(one32);
			nof30.getSubterm().add(vari30);
			HLAnnotation access2cshl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			access2cshl.setStructure(nof30);
			access2cshl.setText("1'(i)");
			ArcHLAPI access2cs = new ArcHLAPI("access2cs", new NameHLAPI("Access2Cs"), accessCS, cs, null,
					new HLAnnotationHLAPI(access2cshl), defaultPage);
			// TestAlone --> NotAlone : 1'(i,j,k)
			NumberConstant one33 = IntegersFactory.eINSTANCE.createNumberConstant();
			one33.setValue(1);
			one33.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari31 = TermsFactory.eINSTANCE.createVariable();
			vari31.setVariableDecl(iDecl.getContainedItem());
			Variable varj22 = TermsFactory.eINSTANCE.createVariable();
			varj22.setVariableDecl(jDecl.getContainedItem());
			Variable vark10 = TermsFactory.eINSTANCE.createVariable();
			vark10.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple29 = TermsFactory.eINSTANCE.createTuple();
			tuple29.getSubterm().add(vari31);
			tuple29.getSubterm().add(varj22);
			tuple29.getSubterm().add(vark10);
			NumberOf nof31 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof31.getSubterm().add(one33);
			nof31.getSubterm().add(tuple29);
			HLAnnotation testAlone2notAlonehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			testAlone2notAlonehl.setStructure(nof31);
			testAlone2notAlonehl.setText("1'(i,j,k)");
			ArcHLAPI testAlone2notAlone = new ArcHLAPI("testAlone2notAlone", new NameHLAPI("TestAlone2NotAlone"),
					testAlone, notAlone, null, new HLAnnotationHLAPI(testAlone2notAlonehl), defaultPage);
			// TestAlone -- > Alone1 : 1'(i,j,k)
			NumberConstant one34 = IntegersFactory.eINSTANCE.createNumberConstant();
			one34.setValue(1);
			one34.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari32 = TermsFactory.eINSTANCE.createVariable();
			vari32.setVariableDecl(iDecl.getContainedItem());
			Variable varj23 = TermsFactory.eINSTANCE.createVariable();
			varj23.setVariableDecl(jDecl.getContainedItem());
			Variable vark11 = TermsFactory.eINSTANCE.createVariable();
			vark11.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple30 = TermsFactory.eINSTANCE.createTuple();
			tuple30.getSubterm().add(vari32);
			tuple30.getSubterm().add(varj23);
			tuple30.getSubterm().add(vark11);
			NumberOf nof32 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof32.getSubterm().add(one34);
			nof32.getSubterm().add(tuple30);
			HLAnnotation testAlone2alone1hl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			testAlone2alone1hl.setStructure(nof32);
			testAlone2alone1hl.setText("1'(i,j,k)");
			ArcHLAPI testAlone2alone1 = new ArcHLAPI("testAlone2alone1", new NameHLAPI("TestAlone2Alone1"), testAlone,
					alone1, null, new HLAnnotationHLAPI(testAlone2alone1hl), defaultPage);
			// NoIdentity --> TestAlone : 1'(i,j,k)
			NumberConstant one35 = IntegersFactory.eINSTANCE.createNumberConstant();
			one35.setValue(1);
			one35.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari33 = TermsFactory.eINSTANCE.createVariable();
			vari33.setVariableDecl(iDecl.getContainedItem());
			Variable varj24 = TermsFactory.eINSTANCE.createVariable();
			varj24.setVariableDecl(jDecl.getContainedItem());
			Variable vark12 = TermsFactory.eINSTANCE.createVariable();
			vark12.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple31 = TermsFactory.eINSTANCE.createTuple();
			tuple31.getSubterm().add(vari33);
			tuple31.getSubterm().add(varj24);
			tuple31.getSubterm().add(vark12);
			NumberOf nof33 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof33.getSubterm().add(one35);
			nof33.getSubterm().add(tuple31);
			HLAnnotation noIdentity2testAlonehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			noIdentity2testAlonehl.setStructure(nof33);
			noIdentity2testAlonehl.setText("1'(i,j,k)");
			ArcHLAPI noIdentity2testAlone = new ArcHLAPI("noIdentity2testAlone", new NameHLAPI("noIdentity2testAlone"),
					noIdentity, testAlone, null, new HLAnnotationHLAPI(noIdentity2testAlonehl), defaultPage);
			// IsEndLoop --> Loop : 1'(i,j,k)
			NumberConstant one36 = IntegersFactory.eINSTANCE.createNumberConstant();
			one36.setValue(1);
			one36.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari34 = TermsFactory.eINSTANCE.createVariable();
			vari34.setVariableDecl(iDecl.getContainedItem());
			Variable varj25 = TermsFactory.eINSTANCE.createVariable();
			varj25.setVariableDecl(jDecl.getContainedItem());
			Variable vark13 = TermsFactory.eINSTANCE.createVariable();
			vark13.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple32 = TermsFactory.eINSTANCE.createTuple();
			tuple32.getSubterm().add(vari34);
			tuple32.getSubterm().add(varj25);
			tuple32.getSubterm().add(vark13);
			NumberOf nof34 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof34.getSubterm().add(one36);
			nof34.getSubterm().add(tuple32);
			HLAnnotation isEndLoop2Loophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			isEndLoop2Loophl.setStructure(nof34);
			isEndLoop2Loophl.setText("1'(i,j,k)");
			ArcHLAPI isEndLoop2Loop = new ArcHLAPI("isEndLoop2Loop", new NameHLAPI("IsEndLoop2Loop"), isEndLoop, loop,
					null, new HLAnnotationHLAPI(isEndLoop2Loophl), defaultPage);
			// IsEndLoop --> EndLoop : 1'(i,j,N)
			NumberConstant one37 = IntegersFactory.eINSTANCE.createNumberConstant();
			one37.setValue(1);
			one37.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari35 = TermsFactory.eINSTANCE.createVariable();
			vari35.setVariableDecl(iDecl.getContainedItem());
			Variable varj26 = TermsFactory.eINSTANCE.createVariable();
			varj26.setVariableDecl(jDecl.getContainedItem());
			UserOperator uo11 = TermsFactory.eINSTANCE.createUserOperator();
			uo11.setDeclaration(procEnumConsts.get(instanceSize).getContainedItem());
			Tuple tuple33 = TermsFactory.eINSTANCE.createTuple();
			tuple33.getSubterm().add(vari35);
			tuple33.getSubterm().add(varj26);
			tuple33.getSubterm().add(uo11);
			NumberOf nof35 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof35.getSubterm().add(one37);
			nof35.getSubterm().add(tuple33);
			HLAnnotation isEndLoop2endLoophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			isEndLoop2endLoophl.setStructure(nof35);
			isEndLoop2endLoophl.setText("1'(i,j,k)");
			ArcHLAPI isEndLoop2endLoop = new ArcHLAPI("isEndLoop2endLoop", new NameHLAPI("IsEndLoop2EndLoop"),
					isEndLoop, endLoop, null, new HLAnnotationHLAPI(isEndLoop2endLoophl), defaultPage);
			// Identity --> IsEndLoop : 1'(i,j,i)
			NumberConstant one38 = IntegersFactory.eINSTANCE.createNumberConstant();
			one38.setValue(1);
			one38.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari36 = TermsFactory.eINSTANCE.createVariable();
			vari36.setVariableDecl(iDecl.getContainedItem());
			Variable varj27 = TermsFactory.eINSTANCE.createVariable();
			varj27.setVariableDecl(jDecl.getContainedItem());
			Variable vari37 = TermsFactory.eINSTANCE.createVariable();
			vari37.setVariableDecl(iDecl.getContainedItem());
			Tuple tuple34 = TermsFactory.eINSTANCE.createTuple();
			tuple34.getSubterm().add(vari36);
			tuple34.getSubterm().add(varj27);
			tuple34.getSubterm().add(vari37);
			NumberOf nof36 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof36.getSubterm().add(one38);
			nof36.getSubterm().add(tuple34);
			HLAnnotation identity2isEndLoophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			identity2isEndLoophl.setStructure(nof36);
			identity2isEndLoophl.setText("1'(i,j,i)");
			ArcHLAPI identity2isEndLoop = new ArcHLAPI("identity2isEndLoop", new NameHLAPI("Identity2EsEndLoop"),
					identity, isEndLoop, null, new HLAnnotationHLAPI(identity2isEndLoophl), defaultPage);
			// Alone1 --> isEndLoop 1'(i,j,k)
			NumberConstant one39 = IntegersFactory.eINSTANCE.createNumberConstant();
			one39.setValue(1);
			one39.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari38 = TermsFactory.eINSTANCE.createVariable();
			vari38.setVariableDecl(iDecl.getContainedItem());
			Variable varj28 = TermsFactory.eINSTANCE.createVariable();
			varj28.setVariableDecl(jDecl.getContainedItem());
			Variable vark14 = TermsFactory.eINSTANCE.createVariable();
			vark14.setVariableDecl(kDecl.getContainedItem());
			Tuple tuple35 = TermsFactory.eINSTANCE.createTuple();
			tuple35.getSubterm().add(vari38);
			tuple35.getSubterm().add(varj28);
			tuple35.getSubterm().add(vark14);
			NumberOf nof37 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof37.getSubterm().add(one39);
			nof37.getSubterm().add(tuple35);
			HLAnnotation alone12isEndLoophl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			alone12isEndLoophl.setStructure(nof37);
			alone12isEndLoophl.setText("1'(i,j,k)");
			ArcHLAPI alone12isEndLoop = new ArcHLAPI("alone12isEndLoop", new NameHLAPI("Alone12IsEndLoop"), alone1,
					isEndLoop, null, new HLAnnotationHLAPI(alone12isEndLoophl), defaultPage);
			// WantSection --> Alone1 : 1'(k,F)
			NumberConstant one40 = IntegersFactory.eINSTANCE.createNumberConstant();
			one40.setValue(1);
			one40.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vark15 = TermsFactory.eINSTANCE.createVariable();
			vark15.setVariableDecl(kDecl.getContainedItem());
			UserOperator uo12 = TermsFactory.eINSTANCE.createUserOperator();
			uo12.setDeclaration(falseConst.getContainedItem());
			Tuple tuple36 = TermsFactory.eINSTANCE.createTuple();
			tuple36.getSubterm().add(vark15);
			tuple36.getSubterm().add(uo12);
			NumberOf nof38 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof38.getSubterm().add(one40);
			nof38.getSubterm().add(tuple36);
			HLAnnotation wantSection2alone1hl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			wantSection2alone1hl.setStructure(nof38);
			wantSection2alone1hl.setText("1'(k,F)");
			ArcHLAPI wantSection2alone1 = new ArcHLAPI("wantSection2alone1", new NameHLAPI("wantSection2Alone1"),
					wantSection, alone1, null, new HLAnnotationHLAPI(wantSection2alone1hl), defaultPage);
			// Alone1 --> WantSection : 1'(k, F)
			NumberConstant one41 = IntegersFactory.eINSTANCE.createNumberConstant();
			one41.setValue(1);
			one41.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vark16 = TermsFactory.eINSTANCE.createVariable();
			vark16.setVariableDecl(kDecl.getContainedItem());
			UserOperator uo13 = TermsFactory.eINSTANCE.createUserOperator();
			uo13.setDeclaration(falseConst.getContainedItem());
			Tuple tuple37 = TermsFactory.eINSTANCE.createTuple();
			tuple37.getSubterm().add(vark16);
			tuple37.getSubterm().add(uo13);
			NumberOf nof39 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof39.getSubterm().add(one41);
			nof39.getSubterm().add(tuple37);
			HLAnnotation alone12wantSectionhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			alone12wantSectionhl.setStructure(nof39);
			alone12wantSectionhl.setText("1'(k,F)");
			ArcHLAPI alone12wantSection = new ArcHLAPI("alone12wantSection", new NameHLAPI("Alone12WantSection"),
					alone1, wantSection, null, new HLAnnotationHLAPI(alone12wantSectionhl), defaultPage);
			// WantSection --> NotAlone : 1'(k,T)
			NumberConstant one42 = IntegersFactory.eINSTANCE.createNumberConstant();
			one42.setValue(1);
			one42.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vark17 = TermsFactory.eINSTANCE.createVariable();
			vark17.setVariableDecl(kDecl.getContainedItem());
			UserOperator uo14 = TermsFactory.eINSTANCE.createUserOperator();
			uo14.setDeclaration(trueConst.getContainedItem());
			Tuple tuple38 = TermsFactory.eINSTANCE.createTuple();
			tuple38.getSubterm().add(vark17);
			tuple38.getSubterm().add(uo14);
			NumberOf nof40 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof40.getSubterm().add(one42);
			nof40.getSubterm().add(tuple38);
			HLAnnotation wantSection2notAlonehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			wantSection2notAlonehl.setStructure(nof40);
			wantSection2notAlonehl.setText("1'(k,T)");
			ArcHLAPI wantSection2notAlone = new ArcHLAPI("wantSection2notAlone", new NameHLAPI("WantSection2NotAlone"),
					wantSection, notAlone, null, new HLAnnotationHLAPI(wantSection2notAlonehl), defaultPage);
			// NotAlone --> WantSection : 1'(k,T)
			NumberConstant one43 = IntegersFactory.eINSTANCE.createNumberConstant();
			one43.setValue(1);
			one43.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vark18 = TermsFactory.eINSTANCE.createVariable();
			vark18.setVariableDecl(kDecl.getContainedItem());
			UserOperator uo15 = TermsFactory.eINSTANCE.createUserOperator();
			uo15.setDeclaration(trueConst.getContainedItem());
			Tuple tuple39 = TermsFactory.eINSTANCE.createTuple();
			tuple39.getSubterm().add(vark18);
			tuple39.getSubterm().add(uo15);
			NumberOf nof41 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof41.getSubterm().add(one43);
			nof41.getSubterm().add(tuple39);
			HLAnnotation notAlone2wantSectionhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			notAlone2wantSectionhl.setStructure(nof41);
			notAlone2wantSectionhl.setText("1'(k,T)");
			ArcHLAPI notAlone2wantSection = new ArcHLAPI("notAlone2wantSection", new NameHLAPI("NotAlone2WantSection"),
					notAlone, wantSection, null, new HLAnnotationHLAPI(notAlone2wantSectionhl), defaultPage);

		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}
	}
}
