package fr.lip6.mcc.pnml.generators;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.CyclicEnumerationsFactory;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.Predecessor;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.Add;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

@Preamble(
		author = "Lom",
		date = "19/02/2011",
		description = "Manual construction of the Petri net (ISO/IEC 15909-2) of the  Philo model. \n",
		provider = "Emmanuel Paviot-Adet."
)
public class Philo extends RootGenerator implements PnmlGenerator {
	
	/**
	 * Root doc Class for the net
	 */
	protected HLAPIRootClass philoDoc;
	
	public Philo() {
		super();
	}

	@Override
	public void generate2Pnml(String helenaFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Philosophers instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Philosophers instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done : Philosophers instance " + instanceSize);
	}
	
	protected void createModel(int instanceSize) {
		philoDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("philoDoc" + instanceSize, philoDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("philoNet", PNTypeHLAPI.SYMNET, new NameHLAPI("Philosophers"), (PetriNetDocHLAPI)philoDoc);
			PageHLAPI defaultPage = new PageHLAPI("philoDefaultPage", net);
			/* Net declarations */
			
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class Philo is 1..2
			CyclicEnumerationHLAPI philoEnum = new CyclicEnumerationHLAPI();
			for (int i = 1; i <= instanceSize; i++){
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("Id" + String.valueOf(i), String.valueOf(i));
				philoEnum.addElementsHLAPI(oneConstant);
			}
			
			NamedSortHLAPI philoName = new NamedSortHLAPI("philo", "Philo", philoEnum, declarations);
			// Var x in Philo 
			VariableDeclHLAPI xDecl = new VariableDeclHLAPI("varx","x", new UserSortHLAPI(philoName), declarations);
			
			/* Places */
			// Place Think; {dom: Philo; mark: Philo.all}
			PlaceHLAPI think = new PlaceHLAPI("think", new NameHLAPI("Think"), null, 
					new TypeHLAPI(null, "Philo", new UserSortHLAPI(philoName)), new HLMarkingHLAPI(null, "Philo.all",
							new AllHLAPI(new UserSortHLAPI(philoName))), defaultPage);
			// Place Fork; {dom:Philo, mark: Philo.all}
			PlaceHLAPI fork = new PlaceHLAPI("fork", new NameHLAPI("Fork"), null, 
					new TypeHLAPI(null, "Philo", new UserSortHLAPI(philoName)), new HLMarkingHLAPI(null, "Philo.all",
							new AllHLAPI(new UserSortHLAPI(philoName))), defaultPage);
			// Place Catch1; {dom: Philo}
			PlaceHLAPI catche1 = new PlaceHLAPI("catch1", new NameHLAPI("Catch1"), null, 
					new TypeHLAPI(null, "Philo", new UserSortHLAPI(philoName)), null, defaultPage);
			// Place Catch2; {dom: Philo}
			PlaceHLAPI catche2 = new PlaceHLAPI("catch2", new NameHLAPI("Catch2"), null, 
					new TypeHLAPI(null, "Philo", new UserSortHLAPI(philoName)), null, defaultPage);
			// Place Eat: {dom: Philo}
			PlaceHLAPI eat = new PlaceHLAPI("eat", new NameHLAPI("Eat"), null, 
					new TypeHLAPI(null, "Philo", new UserSortHLAPI(philoName)), null, defaultPage);
			
			/* Transitions */
			// FF1a
			TransitionHLAPI ff1a = new TransitionHLAPI("ff1a", new NameHLAPI("FF1a"), null, null, defaultPage);
			// FF1b
			TransitionHLAPI ff1b = new TransitionHLAPI("ff1b", new NameHLAPI("FF1b"), null, null, defaultPage);
			//FF2a
			TransitionHLAPI ff2a = new TransitionHLAPI("ff2a", new NameHLAPI("FF2a"), null, null, defaultPage);
			// FF1b
			TransitionHLAPI ff2b = new TransitionHLAPI("ff2b", new NameHLAPI("FF2b"), null, null, defaultPage);
			// End
			TransitionHLAPI end = new TransitionHLAPI("end", new NameHLAPI("End"), null, null, defaultPage);
			
			/* Arcs */
			// Think -> FF1a: 1'(x)
			NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
			one0.setValue(1);
			one0.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx0 = TermsFactory.eINSTANCE.createVariable();
			varx0.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof0.getSubterm().add(one0);
			nof0.getSubterm().add(varx0);
			HLAnnotation think2ff1ahl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			think2ff1ahl.setStructure(nof0);
			think2ff1ahl.setText("1'(x)");
			ArcHLAPI think2ff1a = new ArcHLAPI("think2ff1a", new NameHLAPI("Think2FF1a"), think, ff1a, null,
					new HLAnnotationHLAPI(think2ff1ahl), defaultPage);
			// Think -> FF1b: 1'(x)
			NumberConstant one1 = IntegersFactory.eINSTANCE.createNumberConstant();
			one1.setValue(1);
			one1.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx1 = TermsFactory.eINSTANCE.createVariable();
			varx1.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one1);
			nof1.getSubterm().add(varx1);
			HLAnnotation think2ff1bhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			think2ff1bhl.setStructure(nof1);
			think2ff1bhl.setText("1'(x)");
			ArcHLAPI think2ff1b = new ArcHLAPI("think2ff1b", new NameHLAPI("Think2FF1b"), think, ff1b, null,
					new HLAnnotationHLAPI(think2ff1bhl), defaultPage);
			// Fork -> FF1b: 1'(x)
			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx2 = TermsFactory.eINSTANCE.createVariable();
			varx2.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one2);
			nof2.getSubterm().add(varx2);
			HLAnnotation fork2ff1bhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			fork2ff1bhl.setStructure(nof2);
			fork2ff1bhl.setText("1'(x)");
			ArcHLAPI fork2ff1b = new ArcHLAPI("fork2ff1b", new NameHLAPI("Fork2FF1b"), fork, ff1b, null,
					new HLAnnotationHLAPI(fork2ff1bhl), defaultPage);
			// Fork to FF1a: 1'(x--1)
			/*NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());*/
			Variable varx3 = TermsFactory.eINSTANCE.createVariable();
			varx3.setVariableDecl(xDecl.getContainedItem());
			Predecessor pred0 = CyclicEnumerationsFactory.eINSTANCE.createPredecessor();
			pred0.getSubterm().add(varx3);
			//pred0.getSubterm().add(one3);
			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one4);
			nof3.getSubterm().add(pred0);
			HLAnnotation fork2ff1ahl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			fork2ff1ahl.setStructure(nof3);
			fork2ff1ahl.setText("1'(x--1)");
			ArcHLAPI fork2ff1a = new ArcHLAPI("fork2ff1a", new NameHLAPI("Fork2FF1a"), fork, ff1a, null,
					new HLAnnotationHLAPI(fork2ff1ahl), defaultPage);
			
			// Fork to FF2b: 1'(x)
			Variable varx5 = TermsFactory.eINSTANCE.createVariable();
			varx5.setVariableDecl(xDecl.getContainedItem());
			Predecessor pred1 = CyclicEnumerationsFactory.eINSTANCE.createPredecessor();
			pred1.getSubterm().add(varx5);
			//pred1.getSubterm().add(one6);
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(one7);
			nof5.getSubterm().add(pred1);
			HLAnnotation fork2ff2bhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			fork2ff2bhl.setStructure(nof5);
			fork2ff2bhl.setText("1'(x--1)");
			ArcHLAPI fork2ff2b = new ArcHLAPI("fork2ff2b", new NameHLAPI("Fork2FF2b"), fork, ff2b, null,
					new HLAnnotationHLAPI(fork2ff2bhl), defaultPage);
			// Fork to FF2a: 1'(x--1)
/*			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());*/
			
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx4 = TermsFactory.eINSTANCE.createVariable();
			varx4.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one5);
			nof4.getSubterm().add(varx4);
			HLAnnotation fork2ff2ahl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			fork2ff2ahl.setStructure(nof4);
			fork2ff2ahl.setText("1'(x)");
			ArcHLAPI fork2ff2a = new ArcHLAPI("fork2ff2a", new NameHLAPI("Fork2FF2a"), fork, ff2a, null,
					new HLAnnotationHLAPI(fork2ff2ahl), defaultPage);
			
			
			// Catch1 to FF2a: 1'(x)
			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx6 = TermsFactory.eINSTANCE.createVariable();
			varx6.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(one8);
			nof6.getSubterm().add(varx6);
			HLAnnotation catch12ff2ahl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			catch12ff2ahl.setStructure(nof6);
			catch12ff2ahl.setText("1'(x)");
			ArcHLAPI catch12ff2a = new ArcHLAPI("catch12ff2a", new NameHLAPI("Catch12FF2a"), catche1, ff2a, null,
					new HLAnnotationHLAPI(catch12ff2ahl), defaultPage);
			// Catch2 to FF2b: 1'(x)
			NumberConstant one9 = IntegersFactory.eINSTANCE.createNumberConstant();
			one9.setValue(1);
			one9.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx7 = TermsFactory.eINSTANCE.createVariable();
			varx7.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(one9);
			nof7.getSubterm().add(varx7);
			HLAnnotation catch22ff2bhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			catch22ff2bhl.setStructure(nof7);
			catch22ff2bhl.setText("1'(x)");
			ArcHLAPI catch22ff2b = new ArcHLAPI("catch22ff2b", new NameHLAPI("Catch22FF2b"), catche2, ff2b, null,
					new HLAnnotationHLAPI(catch22ff2bhl), defaultPage);
			// Eat to End: 1'(x)
			NumberConstant one10 = IntegersFactory.eINSTANCE.createNumberConstant();
			one10.setValue(1);
			one10.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx8 = TermsFactory.eINSTANCE.createVariable();
			varx8.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one10);
			nof8.getSubterm().add(varx8);
			HLAnnotation eat2endhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			eat2endhl.setStructure(nof8);
			eat2endhl.setText("1'(x)");
			ArcHLAPI eat2end = new ArcHLAPI("eat2end", new NameHLAPI("Eat2End"), eat, end, null,
					new HLAnnotationHLAPI(eat2endhl), defaultPage);
			// End to Think: 1'(x)
			NumberConstant one11 = IntegersFactory.eINSTANCE.createNumberConstant();
			one11.setValue(1);
			one11.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx9 = TermsFactory.eINSTANCE.createVariable();
			varx9.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof9 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof9.getSubterm().add(one11);
			nof9.getSubterm().add(varx9);
			HLAnnotation end2thinkhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			end2thinkhl.setStructure(nof9);
			end2thinkhl.setText("1'(x)");
			ArcHLAPI end2think = new ArcHLAPI("end2think", new NameHLAPI("End2Think"), end, think, null,
					new HLAnnotationHLAPI(end2thinkhl), defaultPage);
			// FF1a to Catch1: 1'(x)
			NumberConstant one12 = IntegersFactory.eINSTANCE.createNumberConstant();
			one12.setValue(1);
			one12.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx10 = TermsFactory.eINSTANCE.createVariable();
			varx10.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof10 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof10.getSubterm().add(one12);
			nof10.getSubterm().add(varx10);
			HLAnnotation ff1a2catch1hl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			ff1a2catch1hl.setStructure(nof10);
			ff1a2catch1hl.setText("1'(x)");
			ArcHLAPI ff1a2catch1 = new ArcHLAPI("ff1a2catch1", new NameHLAPI("FF1a2Catch1"), ff1a, catche1, null,
					new HLAnnotationHLAPI(ff1a2catch1hl), defaultPage);
			// FF1b to Catch2: 1'(x)
			NumberConstant one13 = IntegersFactory.eINSTANCE.createNumberConstant();
			one13.setValue(1);
			one13.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx11 = TermsFactory.eINSTANCE.createVariable();
			varx11.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof11 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof11.getSubterm().add(one13);
			nof11.getSubterm().add(varx11);
			HLAnnotation ff1b2catch2hl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			ff1b2catch2hl.setStructure(nof11);
			ff1b2catch2hl.setText("1'(x)");
			ArcHLAPI ff1b2catch2 = new ArcHLAPI("ff1b2catch2", new NameHLAPI("FF1b2Catch2"), ff1b, catche2, null,
					new HLAnnotationHLAPI(ff1b2catch2hl), defaultPage);
			// FF2a to Eat: 1'(x)
			NumberConstant one14 = IntegersFactory.eINSTANCE.createNumberConstant();
			one14.setValue(1);
			one14.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx12 = TermsFactory.eINSTANCE.createVariable();
			varx12.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof12 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof12.getSubterm().add(one14);
			nof12.getSubterm().add(varx12);
			HLAnnotation ff2a2eathl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			ff2a2eathl.setStructure(nof12);
			ff2a2eathl.setText("1'(x)");
			ArcHLAPI ff2a2eat = new ArcHLAPI("ff2a2eat", new NameHLAPI("FF2a2Eat"), ff2a, eat, null,
					new HLAnnotationHLAPI(ff2a2eathl), defaultPage);
			// FF2b to Eat: 1'(x)
			NumberConstant one15 = IntegersFactory.eINSTANCE.createNumberConstant();
			one15.setValue(1);
			one15.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx13 = TermsFactory.eINSTANCE.createVariable();
			varx13.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof13 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof13.getSubterm().add(one15);
			nof13.getSubterm().add(varx13);
			HLAnnotation ff2b2eathl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			ff2b2eathl.setStructure(nof13);
			ff2b2eathl.setText("1'(x)");
			ArcHLAPI ff2b2eat = new ArcHLAPI("ff2b2eat", new NameHLAPI("FF2b2Eat"), ff2b, eat, null,
					new HLAnnotationHLAPI(ff2b2eathl), defaultPage);
			// End to Fork: 1'(x) + 1'(x--1)
			NumberConstant one16 = IntegersFactory.eINSTANCE.createNumberConstant();
			one16.setValue(1);
			one16.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx14 = TermsFactory.eINSTANCE.createVariable();
			varx14.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof14 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof14.getSubterm().add(one16);
			nof14.getSubterm().add(varx14);
			Variable varx15 = TermsFactory.eINSTANCE.createVariable();
			varx15.setVariableDecl(xDecl.getContainedItem());
			Predecessor pred2 = CyclicEnumerationsFactory.eINSTANCE.createPredecessor();
			pred2.getSubterm().add(varx15);
			NumberConstant one17 = IntegersFactory.eINSTANCE.createNumberConstant();
			one17.setValue(1);
			one17.setType(IntegersFactory.eINSTANCE.createPositive());
			NumberOf nof15 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof15.getSubterm().add(one17);
			nof15.getSubterm().add(pred2);
			Add add0 = MultisetsFactory.eINSTANCE.createAdd();
			add0.getSubterm().add(nof14);
			add0.getSubterm().add(nof15);
			HLAnnotation end2forkhl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			end2forkhl.setStructure(add0);
			end2forkhl.setText("1'(x) + 1'(x--1)");
			ArcHLAPI end2fork = new ArcHLAPI("end2fork", new NameHLAPI("End2Fork"), end, fork, null,
					new HLAnnotationHLAPI(end2forkhl), defaultPage);	
		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}
	}
}
