package fr.lip6.mcc.pnml.generators;

import java.io.IOException;

import org.apache.commons.logging.Log;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.move.pnml.framework.general.PnmlExport;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.ModelRepository;
import fr.lip6.move.pnml.framework.utils.exception.BadFileFormatException;
import fr.lip6.move.pnml.framework.utils.exception.OCLValidationFailed;
import fr.lip6.move.pnml.framework.utils.exception.OtherException;
import fr.lip6.move.pnml.framework.utils.exception.UnhandledNetType;
import fr.lip6.move.pnml.framework.utils.exception.ValidationFailedException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.framework.utils.logging.LogMaster;

@Preamble(
		author = "Lom",
		date = "11/02/2011",
		description = "Root class to derive from in order to implement \n" +
		    " different transformation strategies from proprietary to PNML",
		provider = "Lom"
)
public abstract class RootGenerator {
	/**
	 * Retrieves a model repository to store PNML models.
	 */
	protected static final ModelRepository MR = ModelRepository.getInstance();
	/**
	 * Create an instance of Log object name parameter; used to know who is
	 * calling the logger like "import" or "export".
	 */
	protected static final Log JOURNAL = LogMaster.giveLogger("PNMLGenerationJournal");
	private static final PnmlExport PEX = new PnmlExport("gmlcustom.xml");
	
	public RootGenerator() {
		super();
	}
	
	protected abstract void createModel(int instanceSize);

	/**
	 * Exports each current PNML model from the PNML repository into a PNML file.
	 * 
	 * @param pnmlFileOut
	 *            the output PNML file.
	 * @throws CamiException
	 *             CamiException
	 */
	protected static void exportToPNML(final String pnmlFileOut) throws MCCException {
		MR.setPrettyPrintStatus(true);
		try {
			final HLAPIRootClass roc = MR.getCurrentHLAPIRootClass();
			 PEX.disableOclChecking();
			 PEX.disableGrammarValidation();
			PEX.exportObject(roc, pnmlFileOut);
			MR.destroyCurrentWorkspace();
		} catch (final VoidRepositoryException vre) {
			JOURNAL.error("exportToPnml: VoidRepositoryException");
			throw new MCCException(vre.getMessage());
		} catch (final UnhandledNetType une) {
			JOURNAL.error("exportToPnml: UnhandledNetType exception");
			throw new MCCException(une.getMessage());
		} catch (final OCLValidationFailed ove) {
			JOURNAL.error("exportToPnml: OCLValidationFailed");
			throw new MCCException(ove.getMessage());
		} catch (final IOException ioe) {
			JOURNAL.error("exportToPnml: IOException");
			throw new MCCException(ioe.getMessage());
		} catch (final ValidationFailedException vfe) {
			JOURNAL.error("exportToPnml: ValidationFailedException");
			throw new MCCException(vfe.getMessage());
		} catch (final BadFileFormatException bfe) {
			JOURNAL.error("exportToPnml: BadFileFormatException");
			throw new MCCException(bfe.getMessage());
		} catch (final OtherException oe) {
			JOURNAL.error("exportToPnml: undefined exception");
			throw new MCCException(oe.getMessage());
		}

	}
}
