package fr.lip6.mcc.pnml.generators;

import java.util.HashMap;

import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.booleans.And;
import fr.lip6.move.pnml.symmetricnet.booleans.BooleansFactory;
import fr.lip6.move.pnml.symmetricnet.booleans.Inequality;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.CyclicEnumerationsFactory;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.Predecessor;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.Successor;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.Condition;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ConditionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.Add;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AddHLAPI;
import fr.lip6.move.pnml.symmetricnet.partitions.GreaterThan;
import fr.lip6.move.pnml.symmetricnet.partitions.PartitionElementOf;
import fr.lip6.move.pnml.symmetricnet.partitions.PartitionsFactory;
import fr.lip6.move.pnml.symmetricnet.partitions.hlapi.PartitionElementHLAPI;
import fr.lip6.move.pnml.symmetricnet.partitions.hlapi.PartitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Tuple;
import fr.lip6.move.pnml.symmetricnet.terms.UserOperator;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserOperatorHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

public class TokenRing extends RootGenerator implements PnmlGenerator {

	/**
	 * Parameter value for the Token Ring instances. From 1 to 10, then 15, 20, 30, 50 (0..N)
	 * 30, 50. Just need to change N value and the name of the output model
	 * accordingly.
	 */
	//protected static final int instanceSize = 1;

	/**
	 * Root doc Class for the net
	 */
	protected HLAPIRootClass trDoc;

	public TokenRing() {
		super();
	}
	
	@Override
	public void generate2Pnml(String proprietaryFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Token Ring instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Token Ring instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done : Token Ring instance " + instanceSize);
	}

	protected void createModel(int instanceSize) {
		trDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("tokenringDoc"+instanceSize, trDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("tokenRing", PNTypeHLAPI.SYMNET, new NameHLAPI("Token Ring"),
					(PetriNetDocHLAPI) trDoc);
			PageHLAPI defaultPage = new PageHLAPI("tokenringDefaultPage", net);

			/* Net declarations */
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class Process is 0..N
			CyclicEnumerationHLAPI procEnum = new CyclicEnumerationHLAPI();
			NamedSortHLAPI procName = new NamedSortHLAPI("process", "Process", procEnum, declarations);
			// Partition over Process : {{0}, {1..N}}
			PartitionHLAPI procPartition = new PartitionHLAPI("processpartition", "Partition over Process",
					new UserSortHLAPI(procName), declarations);
			HashMap<Integer, FEConstantHLAPI> procEnumConsts = new HashMap<Integer, FEConstantHLAPI>();
			// Process feconstant 0 in particular.
			FEConstantHLAPI zeroConstant = new FEConstantHLAPI("process" + String.valueOf(0), String.valueOf(0));
			procEnum.addElementsHLAPI(zeroConstant);
			procEnumConsts.put(0, zeroConstant);
			PartitionElementHLAPI zeroPe = new PartitionElementHLAPI("partitionelement" + String.valueOf(0),
					String.valueOf(0), procPartition);
			UserOperator uoZero = TermsFactory.eINSTANCE.createUserOperator();
			uoZero.setDeclaration(zeroConstant.getContainedItem());
			zeroPe.addPartitionelementconstantsHLAPI(new UserOperatorHLAPI(uoZero));
			// Partition element for the rest
			PartitionElementHLAPI restPe = new PartitionElementHLAPI("partitionelement1ToN",
					"1N", procPartition);
			
			for (int i = 1; i <= instanceSize; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("process" + String.valueOf(i), String.valueOf(i));
				procEnum.addElementsHLAPI(oneConstant);
				procEnumConsts.put(i, oneConstant);
				UserOperator uo0 = TermsFactory.eINSTANCE.createUserOperator();
				uo0.setDeclaration(oneConstant.getContainedItem());
				restPe.addPartitionelementconstantsHLAPI(new UserOperatorHLAPI(uo0));
			}

			// Domain Couple is <Process, Process>
			ProductSortHLAPI couple = new ProductSortHLAPI();
			couple.addElementSortHLAPI(new UserSortHLAPI(procName));
			couple.addElementSortHLAPI(new UserSortHLAPI(procName));
			NamedSortHLAPI coupleName = new NamedSortHLAPI("couple", "Couple", couple, declarations);
			// Var i, x, y in Process
			VariableDeclHLAPI iDecl = new VariableDeclHLAPI("vari", "i", new UserSortHLAPI(procName), declarations);
			VariableDeclHLAPI xDecl = new VariableDeclHLAPI("varx", "x", new UserSortHLAPI(procName), declarations);
			VariableDeclHLAPI yDecl = new VariableDeclHLAPI("vary", "y", new UserSortHLAPI(procName), declarations);

			/* Places */
			// State, domain: Couple; mark: {($i,$i) | 0 leq $i leq N}
			Add add = MultisetsFactory.eINSTANCE.createAdd();
			for (int i = 0; i <= instanceSize; i++) {
				UserOperator uo1 = TermsFactory.eINSTANCE.createUserOperator();
				uo1.setDeclaration(procEnumConsts.get(i).getContainedItem());
				UserOperator uo2 = TermsFactory.eINSTANCE.createUserOperator();
				uo2.setDeclaration(procEnumConsts.get(i).getContainedItem());
				Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
				tuple0.getSubterm().add(uo1);
				tuple0.getSubterm().add(uo2);
				NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
				one0.setValue(1);
				one0.setType(IntegersFactory.eINSTANCE.createPositive());
				NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
				nof0.getSubterm().add(one0);
				nof0.getSubterm().add(tuple0);
				add.getSubterm().add(nof0);
			}
			PlaceHLAPI state = new PlaceHLAPI("state", new NameHLAPI("State"), null, new TypeHLAPI(null, "Couple",
					new UserSortHLAPI(coupleName)), new HLMarkingHLAPI(null, "{($i,$i) | 0 leq $i leq N}",
					new AddHLAPI(add)), defaultPage);

			/* Transitions */
			// MainProcess
			TransitionHLAPI mainProcess = new TransitionHLAPI("mainprocess", new NameHLAPI("MainProcess"), null, null,
					defaultPage);
			// OtherProcess {guard: i gt 0 and x ineq y}
			Variable vari0 = TermsFactory.eINSTANCE.createVariable();
			vari0.setVariableDecl(iDecl.getContainedItem());
			UserOperator uo2 = TermsFactory.eINSTANCE.createUserOperator();
			uo2.setDeclaration(procEnumConsts.get(0).getContainedItem());
			PartitionElementOf peo0 = PartitionsFactory.eINSTANCE.createPartitionElementOf();
			peo0.setRefpartition(procPartition.getContainedItem());
			peo0.getSubterm().add(vari0);
			PartitionElementOf peo1 = PartitionsFactory.eINSTANCE.createPartitionElementOf();
			peo1.setRefpartition(procPartition.getContainedItem());
			peo1.getSubterm().add(uo2);
			GreaterThan gtp = PartitionsFactory.eINSTANCE.createGreaterThan();
			gtp.getSubterm().add(peo0);
			gtp.getSubterm().add(peo1);

			Variable varx0 = TermsFactory.eINSTANCE.createVariable();
			varx0.setVariableDecl(xDecl.getContainedItem());
			Variable vary0 = TermsFactory.eINSTANCE.createVariable();
			vary0.setVariableDecl(yDecl.getContainedItem());
			Inequality ineq0 = BooleansFactory.eINSTANCE.createInequality();
			ineq0.getSubterm().add(varx0);
			ineq0.getSubterm().add(vary0);

			And and0 = BooleansFactory.eINSTANCE.createAnd();
			and0.getSubterm().add(gtp);
			and0.getSubterm().add(ineq0);

			Condition otherProcCond = HlcorestructureFactory.eINSTANCE.createCondition();
			otherProcCond.setStructure(and0);
			otherProcCond.setText("[i gt 0 and x ineq y]");
			TransitionHLAPI otherProcess = new TransitionHLAPI("otherprocess", new NameHLAPI("OtherProcess"), null,
					new ConditionHLAPI(otherProcCond), defaultPage);

			/* Arcs */
			// MainProcess --> State : 1'(0, x++1) + 1'(N, x)
			NumberConstant one1 = IntegersFactory.eINSTANCE.createNumberConstant();
			one1.setValue(1);
			one1.setType(IntegersFactory.eINSTANCE.createPositive());
			UserOperator uo3 = TermsFactory.eINSTANCE.createUserOperator();
			uo3.setDeclaration(procEnumConsts.get(0).getContainedItem());
			Variable varx1 = TermsFactory.eINSTANCE.createVariable();
			varx1.setVariableDecl(xDecl.getContainedItem());
			Successor succ0 = CyclicEnumerationsFactory.eINSTANCE.createSuccessor();
			succ0.getSubterm().add(varx1);
			Tuple tuple1 = TermsFactory.eINSTANCE.createTuple();
			tuple1.getSubterm().add(uo3);
			tuple1.getSubterm().add(succ0);
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one1);
			nof1.getSubterm().add(tuple1);

			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			UserOperator uo4 = TermsFactory.eINSTANCE.createUserOperator();
			uo4.setDeclaration(procEnumConsts.get(instanceSize).getContainedItem());
			Variable varx2 = TermsFactory.eINSTANCE.createVariable();
			varx2.setVariableDecl(xDecl.getContainedItem());
			Tuple tuple2 = TermsFactory.eINSTANCE.createTuple();
			tuple2.getSubterm().add(uo4);
			tuple2.getSubterm().add(varx2);
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one2);
			nof2.getSubterm().add(tuple2);

			Add add1 = MultisetsFactory.eINSTANCE.createAdd();
			add1.getSubterm().add(nof1);
			add1.getSubterm().add(nof2);

			HLAnnotation mainproc2statehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			mainproc2statehl.setStructure(add1);
			mainproc2statehl.setText("1'(0, x++1) + 1'(N, x)");
			ArcHLAPI mainproc2state = new ArcHLAPI("mainproc2state", new NameHLAPI("MainProcess2State"), mainProcess,
					state, null, new HLAnnotationHLAPI(mainproc2statehl), defaultPage);

			// State --> Main process = 1'(0, x) + 1'(N,x)
			NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());
			UserOperator uo5 = TermsFactory.eINSTANCE.createUserOperator();
			uo5.setDeclaration(procEnumConsts.get(0).getContainedItem());
			Variable varx3 = TermsFactory.eINSTANCE.createVariable();
			varx3.setVariableDecl(xDecl.getContainedItem());
			Tuple tuple3 = TermsFactory.eINSTANCE.createTuple();
			tuple3.getSubterm().add(uo5);
			tuple3.getSubterm().add(varx3);
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one3);
			nof3.getSubterm().add(tuple3);

			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			UserOperator uo6 = TermsFactory.eINSTANCE.createUserOperator();
			uo6.setDeclaration(procEnumConsts.get(instanceSize).getContainedItem());
			Variable varx4 = TermsFactory.eINSTANCE.createVariable();
			varx4.setVariableDecl(xDecl.getContainedItem());
			Tuple tuple4 = TermsFactory.eINSTANCE.createTuple();
			tuple4.getSubterm().add(uo6);
			tuple4.getSubterm().add(varx4);
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one4);
			nof4.getSubterm().add(tuple4);

			Add add2 = MultisetsFactory.eINSTANCE.createAdd();
			add2.getSubterm().add(nof3);
			add2.getSubterm().add(nof4);

			HLAnnotation state2mainprochl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			state2mainprochl.setStructure(add2);
			state2mainprochl.setText("1'(0, x) + 1'(N,x)");
			ArcHLAPI state2mainproc = new ArcHLAPI("state2mainproc", new NameHLAPI("State2MainProc"), state,
					mainProcess, null, new HLAnnotationHLAPI(state2mainprochl), defaultPage);

			// OtherProcess --> State : 1'(i, y) + 1'(i--1, y)
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari1 = TermsFactory.eINSTANCE.createVariable();
			vari1.setVariableDecl(iDecl.getContainedItem());
			Variable vary1 = TermsFactory.eINSTANCE.createVariable();
			vary1.setVariableDecl(yDecl.getContainedItem());
			Tuple tuple5 = TermsFactory.eINSTANCE.createTuple();
			tuple5.getSubterm().add(vari1);
			tuple5.getSubterm().add(vary1);
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(one5);
			nof5.getSubterm().add(tuple5);

			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari2 = TermsFactory.eINSTANCE.createVariable();
			vari2.setVariableDecl(iDecl.getContainedItem());
			Predecessor pred0 = CyclicEnumerationsFactory.eINSTANCE.createPredecessor();
			pred0.getSubterm().add(vari2);
			Variable vary2 = TermsFactory.eINSTANCE.createVariable();
			vary2.setVariableDecl(yDecl.getContainedItem());
			Tuple tuple6 = TermsFactory.eINSTANCE.createTuple();
			tuple6.getSubterm().add(pred0);
			tuple6.getSubterm().add(vary2);
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(one6);
			nof6.getSubterm().add(tuple6);

			Add add3 = MultisetsFactory.eINSTANCE.createAdd();
			add3.getSubterm().add(nof5);
			add3.getSubterm().add(nof6);

			HLAnnotation otherproc2statehl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			otherproc2statehl.setStructure(add3);
			otherproc2statehl.setText("1'(i, y) + 1'(i--1, y)");
			ArcHLAPI ptherproc2state = new ArcHLAPI("otherproc2state", new NameHLAPI("OtherProcess2State"),
					otherProcess, state, null, new HLAnnotationHLAPI(otherproc2statehl), defaultPage);

			// State --> OtherProcess: 1'(i, x) + 1'(i--1, y)
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari3 = TermsFactory.eINSTANCE.createVariable();
			vari3.setVariableDecl(iDecl.getContainedItem());
			Variable varx5 = TermsFactory.eINSTANCE.createVariable();
			varx5.setVariableDecl(xDecl.getContainedItem());
			Tuple tuple7 = TermsFactory.eINSTANCE.createTuple();
			tuple7.getSubterm().add(vari3);
			tuple7.getSubterm().add(varx5);
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(one7);
			nof7.getSubterm().add(tuple7);

			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vari4 = TermsFactory.eINSTANCE.createVariable();
			vari4.setVariableDecl(iDecl.getContainedItem());
			Predecessor pred1 = CyclicEnumerationsFactory.eINSTANCE.createPredecessor();
			pred1.getSubterm().add(vari4);
			Variable vary3 = TermsFactory.eINSTANCE.createVariable();
			vary3.setVariableDecl(yDecl.getContainedItem());
			Tuple tuple8 = TermsFactory.eINSTANCE.createTuple();
			tuple8.getSubterm().add(pred1);
			tuple8.getSubterm().add(vary3);
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one8);
			nof8.getSubterm().add(tuple8);

			Add add4 = MultisetsFactory.eINSTANCE.createAdd();
			add4.getSubterm().add(nof7);
			add4.getSubterm().add(nof8);

			HLAnnotation state2otherprochl = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			state2otherprochl.setStructure(add4);
			state2otherprochl.setText("1'(i, x) + 1'(i--1, y)");
			ArcHLAPI state2otherproc = new ArcHLAPI("state2otherproc", new NameHLAPI("State2OtherProcess"), state,
					otherProcess, null, new HLAnnotationHLAPI(state2otherprochl), defaultPage);		
		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}

	}
}
